<?php

declare(strict_types=1);

namespace Drupal\qr_generator\Plugin\rest\resource;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\KeyValueStore\KeyValueFactoryInterface;
use Drupal\Core\KeyValueStore\KeyValueStoreInterface;
use Drupal\Core\Routing\TrustedRedirectResponse;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\qr_generator\Exception\QRCodeExpiredHttpException;
use Drupal\rest\Attribute\RestResource;
use Drupal\rest\Plugin\ResourceBase;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Represents QR Generator Resource records as resources.
 *
 * @DCG
 * The plugin exposes key-value records as REST resources. In order to enable it
 * import the resource configuration into active configuration storage. An
 * example of such configuration can be located in the following file:
 * core/modules/rest/config/optional/rest.resource.entity.node.yml.
 * Alternatively, you can enable it through admin interface provider by REST UI
 * module.
 * @see https://www.drupal.org/project/restui
 *
 * @DCG
 * Notice that this plugin does not provide any validation for the data.
 * Consider creating custom normalizer to validate and normalize the incoming
 * data. It can be enabled in the plugin definition as follows.
 * @code
 *   serialization_class = "Drupal\foo\MyDataStructure",
 * @endcode
 *
 * @DCG
 * For entities, it is recommended to use REST resource plugin provided by
 * Drupal core.
 * @see \Drupal\rest\Plugin\rest\resource\EntityResource
 */
#[RestResource(
  id: 'qr_code_rest_resource',
  label: new TranslatableMarkup('QR Code Rest Resource'),
  uri_paths: [
    'canonical' => '/api/qr-code/{uuid}'
  ],
)]
final class QRRestResource extends ResourceBase
{

  /**
   * The key-value storage.
   */
  private readonly KeyValueStoreInterface $storage;

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    array $serializer_formats,
    LoggerInterface $logger,
    KeyValueFactoryInterface $keyValueFactory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);
    $this->storage = $keyValueFactory->get('qr_generator_rest_resource');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition): self
  {
    return new self(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('keyvalue')
    );
  }

  /**
   * Responds to GET requests.
   */
  public function get($uuid): TrustedRedirectResponse
  {
    // Check if entity exist
    $entities = \Drupal::entityTypeManager()->getStorage('qr_code')->loadByProperties(['uuid' => $uuid]);
    if (!$entities) {
      throw new NotFoundHttpException('QR-Code not found!');
    }
    $entity = reset($entities);
    $expired = $entity->get('field_expires')->getString();

    // Check if qr-code expired
    if ($expired) {
      $timestamp = \Drupal::time()->getCurrentTime(); // website datetime in timestamp
      if (!empty($timestamp)) {
        if ($timestamp >= (int)$entity->get('field_expires')->getString()) {
          throw new QRCodeExpiredHttpException('This QR code has expired!');
        }
      }
    }
    return new TrustedRedirectResponse($entity->get('field_redirect_url')->getString());
  }

  public function permissions()
  {
    // Don't declare any permissions, or return empty array
    return [];
  }

  public function access($operation = 'view', AccountInterface $account = NULL, $return_as_object = FALSE)
  {
    return AccessResult::allowed();
  }
}
